==> glib-web-0.5.67.tgz: package/components/mixins/events.js <==
import Action from "../../action";
import UrlUtils from "../../utils/url";
import TypeUtils from "../../utils/type";

export default {
  data() {
    return {
      // TODO: Consider replacing with an enum, e.g. executing with predefined values ('post', 'get', null)
      // The complexity with this is that 'get' that comes after 'post' needs to show an indicator, e.g.
      // Submit button needs to be disabled when redirecting after form submission
      _isBusy: false,
      _events: [],
      _typingTimer: null,
    };
  },
  computed: {
    _pageBody() {
      return document.getElementById("page_body");
    },
    $isBusy() {
      // Properties starting with $ or _ can only be accessed via $data
      return this.$data._isBusy;
    },
    $isReady() {
      return this.$data._events === null;
    },
  },
  mounted() {
    this._executeIfReady(false);
    this.$mounted();
  },
  updated() {
    this._executeIfReady(true);
    this.$updated();
  },
  destroyed() {
    this.$tearDown();
  },
  watch: {
    // See Utils.http
    "$root.vueApp.isStale": function(val, oldVal) {
      // Make sure $ready() will be called in the next update
      this._mountedUrl = null;
    },
  },
  methods: {
    $href: function(spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;
      if (TypeUtils.isObject(onClick)) {
        switch (onClick.action) {
          case "windows/open-v1":
          case "windows/openWeb-v1":
          case "windows/reload-v1":
            return UrlUtils.htmlUrl(onClick.url);
        }
      }
      return null;
    },
    $onClick: function(explicitEvent, spec) {
      const properties = spec || this.spec;
      const e = explicitEvent || event;

      if (!properties.onClick) {
        return; // Avoid blocking the event for parents.
      }

      // Ignore middle/right clicks
      if (!e.metaKey && !e.ctrlKey && e.button === 0) {
        e.preventDefault();
        e.stopPropagation();
        // Action.execute(properties.onClick, target || event.target, this);
        Action.execute(properties.onClick, null, this);
      }
    },
    $rel: function(spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;
      if (TypeUtils.isObject(onClick)) {
        if (onClick.action === "windows/openWeb-v1") {
          return "nofollow";
        }
      }
      return null;
    },
    // Can be used for optimization, but not needed for now.
    // $onLongPressFunction() {
    //   return this.spec.onLongPress ? this.$onLongPress : null;
    // },
    $onLongPress() {
      GLib.action.execute(this.spec.onLongPress, null, this);
    },
    $addViewportChangeListeners: function(handler) {
      window.addEventListener("load", handler);
      window.addEventListener("resize", handler);
      this._pageBody.addEventListener("scroll", handler);
    },
    $removeViewportChangeListeners: function(handler) {
      window.removeEventListener("load", handler);
      window.removeEventListener("resize", handler);
      this._pageBody.removeEventListener("scroll", handler);
    },
    _executeIfReady(updated) {
      if (updated) {
        if (!this._renderingTheSamePage()) {
          this.$tearDown();
          this._ready();
        }
      } else {
        this._ready();
      }
    },
    _renderingTheSamePage() {
      return this._mountedUrl === window.location.href;
    },
    _ready() {
      this._mountedUrl = window.location.href;
      this._updatedAt = new Date().getTime();

      this.$data._events = [];

      // Has to be executed before $ready()
      this._linkFieldModels();

      this.$ready();

      // Make sure events are dispatched after $ready(). Dispatching them during $ready() causes errors.
      for (const event of this.$data._events) {
        // Execute later to ensure the parent has registered its event listener
        // setTimeout(() => {
        this.$el.dispatchEvent(event);
        // }, 100)
      }
      this.$data._events = null;
    },
    $ready() {
      // To be overridden
    },
    $tearDown() {
      // To be overridden
    },
    $mounted() {
      // To be overridden
    },
    $updated() {
      // To be overridden
    },
    $dispatchEvent(name, data) {
      const event = new Event(name, { bubbles: true });

      // TODO: Deprecate
      event.spec = data;

      event.data = data;

      if (this.$data._events) {
        this.$data._events.push(event);
      } else {
        this.$el.dispatchEvent(event);
      }
    },
    $onEvent(name, handler) {
      this.$el.addEventListener(name, handler);
    },
    $onTyping({ duration = 2000 }) {
      GLib.action.execute(this.spec.onTypeStart, null, this);
      clearTimeout(this.timer);

      this.timer = setTimeout(() => {
        this._typingTimer = null;
        GLib.action.execute(this.spec.onTypeEnd, null, this);
      }, duration);
    },
    $onZero() {
      GLib.action.execute(this.spec.onZero, null, this);
    },
  },
};

==> glib-web-0.5.86.tgz: package/components/mixins/events.js <==
import Action from "../../action";
import UrlUtils from "../../utils/url";
import TypeUtils from "../../utils/type";

export default {
  data() {
    return {
      // TODO: Consider replacing with an enum, e.g. executing with predefined values ('post', 'get', null)
      // The complexity with this is that 'get' that comes after 'post' needs to show an indicator, e.g.
      // Submit button needs to be disabled when redirecting after form submission
      _isBusy: false,
      _events: [],
      _typingTimer: null
    };
  },
  computed: {
    _pageBody() {
      return document.getElementById("page_body");
    },
    $isBusy() {
      // Properties starting with $ or _ can only be accessed via $data
      return this.$data._isBusy;
    },
    $isReady() {
      return this.$data._events === null;
    }
  },
  mounted() {
    this._executeIfReady(false);
    this.$mounted();
  },
  updated() {
    this._executeIfReady(true);
    this.$updated();
  },
  destroyed() {
    this.$tearDown();
  },
  watch: {
    // See Utils.http
    "$root.vueApp.isStale": function(val, oldVal) {
      // Make sure $ready() will be called in the next update
      this._mountedUrl = null;
    }
  },
  methods: {
    $href: function(spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;
      if (TypeUtils.isObject(onClick)) {
        switch (onClick.action) {
          case "windows/open-v1":
          case "windows/openWeb-v1":
          case "windows/reload-v1":
            return UrlUtils.htmlUrl(onClick.url);
        }
      }
      return null;
    },
    $onClick: function(explicitEvent, spec) {
      const properties = spec || this.spec;
      const e = explicitEvent || event;

      if (!properties.onClick) {
        return; // Avoid blocking the event for parents.
      }

      // Ignore middle/right clicks
      if (!e.metaKey && !e.ctrlKey && e.button === 0) {
        e.preventDefault();
        e.stopPropagation();
        // Action.execute(properties.onClick, target || event.target, this);
        Action.execute(properties.onClick, null, this);
      }
    },
    $rel: function(spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;
      if (TypeUtils.isObject(onClick)) {
        if (onClick.action === "windows/openWeb-v1") {
          return "nofollow";
        }
      }
      return null;
    },
    // Can be used for optimization, but not needed for now.
    // $onLongPressFunction() {
    //   return this.spec.onLongPress ? this.$onLongPress : null;
    // },
    $onLongPress() {
      GLib.action.execute(this.spec.onLongPress, null, this);
    },
    $addViewportChangeListeners: function(handler) {
      window.addEventListener("load", handler);
      window.addEventListener("resize", handler);
      this._pageBody.addEventListener("scroll", handler);
    },
    $removeViewportChangeListeners: function(handler) {
      window.removeEventListener("load", handler);
      window.removeEventListener("resize", handler);
      this._pageBody.removeEventListener("scroll", handler);
    },
    _executeIfReady(updated) {
      if (updated) {
        if (!this._renderingTheSamePage()) {
          this.$tearDown();
          this._ready();
        }
      } else {
        this._ready();
      }
    },
    _renderingTheSamePage() {
      return this._mountedUrl === window.location.href;
    },
    _ready() {
      this._mountedUrl = window.location.href;
      this._updatedAt = new Date().getTime();

      this.$data._events = [];

      // Has to be executed before $ready()
      this._linkFieldModels();

      this.$ready();

      // Make sure events are dispatched after $ready(). Dispatching them during $ready() causes errors.
      for (const event of this.$data._events) {
        // Execute later to ensure the parent has registered its event listener
        // setTimeout(() => {
        this.$el.dispatchEvent(event);
        // }, 100)
      }
      this.$data._events = null;
    },
    $ready() {
      // To be overridden
    },
    $tearDown() {
      // To be overridden
    },
    $mounted() {
      // To be overridden
    },
    $updated() {
      // To be overridden
    },
    $dispatchEvent(name, data) {
      const event = new Event(name, { bubbles: true });

      // TODO: Deprecate
      event.spec = data;

      event.data = data;

      if (this.$data._events) {
        this.$data._events.push(event);
      } else {
        this.$el.dispatchEvent(event);
      }
    },
    $onEvent(name, handler) {
      this.$el.addEventListener(name, handler);
    },
    $onTyping({ duration = 2000 }) {
      GLib.action.execute(this.spec.onTypeStart, null, this);
      clearTimeout(this.timer);

      this.timer = setTimeout(() => {
        this._typingTimer = null;
        GLib.action.execute(this.spec.onTypeEnd, null, this);
      }, duration);
    }
    // $onZero() {
    //   GLib.action.execute(this.spec.onZero, null, this);
    // },
  }
};
